@@ -21,14 +21,14 @@
  *
  */
 
-#include "internal.h"
+#include "finch.h"
 #include "gntidle.h"
 #include "gntwm.h"
 
 #include "idle.h"
 
 static time_t
-finch_get_idle_time()
+finch_get_idle_time(void)
 {
 	return gnt_wm_get_idle_time();
 }
