@@ -390,7 +390,7 @@ static void ggp_callback_register_account_ok(PurpleConnection *gc,
 		goto exit_err;
 	}
 
-	purple_debug_info("gg", "register_account_ok: token_id = %d; t = %s\n",
+	purple_debug_info("gg", "register_account_ok: token_id = %s; t = %s\n",
 			token->id, t);
 	h = gg_register3(email, p1, token->id, t, 0);
 	if (h == NULL || !(s = h->data) || !s->success) {
@@ -1324,7 +1324,7 @@ static void ggp_callback_recv(gpointer _gc, gint fd, PurpleInputCondition cond)
 			break;
 		case GG_EVENT_ACK:
 			purple_debug_info("gg",
-				"message sent to: %ld, delivery status=%d, seq=%d\n",
+				"message sent to: %u, delivery status=%d, seq=%d\n",
 				ev->event.ack.recipient, ev->event.ack.status,
 				ev->event.ack.seq);
 			break;
@@ -2088,7 +2088,7 @@ static PurplePluginProtocolInfo prpl_info =
 	OPT_PROTO_REGISTER_NOSCREENNAME,
 	NULL,				/* user_splits */
 	NULL,				/* protocol_options */
-	NO_BUDDY_ICONS,			/* icon_spec */
+	{"png", 32, 32, 96, 96, 0, PURPLE_ICON_SCALE_DISPLAY},	/* icon_spec */
 	ggp_list_icon,			/* list_icon */
 	NULL,				/* list_emblem */
 	ggp_status_text,		/* status_text */
