@@ -191,6 +191,11 @@ struct _JabberStream
 	char *old_uri;
 	int old_length;
 	char *old_track;
+	
+	char *certificate_CN;
+	
+	/* A purple timeout tag for the keepalive */
+	int keepalive_timeout;
 };
 
 typedef gboolean (JabberFeatureEnabled)(JabberStream *js, const gchar *shortname, const gchar *namespace);
